/*---------------------------------------------------------------------------*/
/* File:     DATA.C                                                          */
/* Author:   R. P. Hanson                                                    */
/* Copyright:R. P. Hanson 1996                                               */
/*           Lothar Bittner is now the authorised custodian of the Atari ST  */
/*           version of this program.                                        */
/*---------------------------------------------------------------------------*/
/* Contents: Functions to deal with input and output of data.                */
/*---------------------------------------------------------------------------*/
/* Functions:"Mout_7_bit"     outputs a 7 bit value.                         */
/*           "Mout_8_bit"     splits an 8 bit value by outputting the most   */
/*                            significant bit as the least significant bit   */
/*                            of a single byte, and then outputting the      */
/*                            lower 7 bits as the next value.                */
/*           "ident_out"      outputs the synth's identity string.           */
/*           "start_sysex"    outputs the first 4 bytes of a Yamaha system   */
/*                            exclusive dump.                                */
/*           "out_envelope"   outputs an envelope structure.                 */
/*           "out_voice"      outputs an entire voice structure.             */
/*           "to_8", "to_8_t" builds an 8 bit value from two 7 bit values,   */
/*                            i.e. the inverse to "Mout_8_bit". Both perform */
/*                            the same function on different global arrays.  */
/*           "get_envelope"   gets an envelope structure from input device.  */
/*           "save_voice"     gets a filename from the operator and if valid */
/*                            voice name, saves the current voice to disk.   */
/*           "in_voice"       gets a filename from input device.             */
/*---------------------------------------------------------------------------*/
/* Globals:  "MIDI_channel"   self-explanatory!                              */
/*           "glob_checksum"  holds the system exclusive checksum - it is    */
/*                            set to 0 before data is output and every time  */
/*                            a byte is output, the value of the byte is     */
/*                            added to this value.                           */
/*           "rx_data"        a buffer for received system exclusive data.   */
/*           "tmp_pt", "pt", "pt2"  temporary pointers used when processing  */
/*                            incoming data.                                 */
/*           "fpt"            file pointer for disk operations.              */
/*           "glob_drv"       if set to 1, output to disk, else to MIDI.     */
/*           "ident"          array holding ident string for SY-22 & SY-35.  */
/*           "curr_path"      current path to load and save voices.          */
/*---------------------------------------------------------------------------*/


/*----------*/
/* Includes */
/*----------*/
    #include "types.h"
    #include <string.h>
    #include <osbind.h>
    #include <stdio.h>

/*------------------*/
/* Global variables */
/*------------------*/
    BYTE MIDI_channel ;
    BYTE glob_checksum ;
    BYTE rx_data[0x270] ;
    BYTE *tmp_pt, *pt, *pt2 ;
    FILE *fpt ;
    int  glob_drv ;
    BYTE ident[] = { 'P', 'K', ' ', ' ', '2', '2', '0', '3', 'A', 'E' } ;
    char curr_path[80] = "A:\*.BNK" ;

/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "Mout_7_bit"                                                */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs a single 7 bit value to disk or MIDI. */
/*              If "glob_drv" is set to 1, then the byte is output to disk, */
/*              otherwise it is output to MIDI.                             */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "byte" : byte to be output.                                 */
/*--------------------------------------------------------------------------*/

    Mout_7_bit ( byte )
    BYTE byte ;
    {
        if ( glob_drv == 1 )  fputc ( byte, fpt ) ; /* Output to disk */
        else                  Bconout ( 3, byte ) ; /* Output to MIDI */

        glob_checksum += byte ; /* Increment system exclusive checksum */
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "Mout_7_bit"  */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "Mout_8_bit"                                                */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs an 8 bit value to disk or MIDI. The   */
/*              most significant bit is removed from the 8 bit value, moved */
/*              to the least significant bit, and then output using the     */
/*              function "Mout_7_bit". The low 7 bits are then masked and   */
/*              output using "Mout_7_bit".                                  */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "byte" : byte to be output.                                 */
/*--------------------------------------------------------------------------*/

    Mout_8_bit ( byte )
    BYTE byte ;
    {
        Mout_7_bit ( (byte >> 7) & 1 ) ;
        Mout_7_bit ( byte & 0x7F ) ;
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "Mout_8_bit"  */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "ident_out"                                                 */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs the SY-22/SY-35 ident string as held  */
/*              in global array "ident".                                    */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    ident_out ()
    {
        int i ;
        for ( i = 0 ; i < 10 ; i++ )  Mout_7_bit ( ident[i] ) ;
    }

/*---------------------------------*/
/*  END OF FUNCTION:  "ident_out"  */
/*---------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "start_sysex"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs Yamaha system exclusive start bytes.  */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "mask" : this value is masked with global "MIDI_channel".   */
/*                       Its value depends on whether the system exclusive  */
/*                       message is to be voice data or a request.          */
/*--------------------------------------------------------------------------*/

    start_sysex ( mask )
    BYTE mask ;
    {
        Mout_7_bit ( 0xF0 ) ;
        Mout_7_bit ( 0x43 ) ;
        Mout_7_bit ( MIDI_channel | mask ) ;
        Mout_7_bit ( 0x7E ) ;
    }

/*-----------------------------------*/
/*  END OF FUNCTION:  "start_sysex"  */
/*-----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "out_envelope"                                              */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs an envelope structure.                */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "env" : structure holding the envelope to be output.        */
/*--------------------------------------------------------------------------*/

    out_envelope ( env )
    env_type *env ;
    {
        int j ;

        Mout_7_bit ( env->delay_switch ) ;
        Mout_7_bit ( env->attack_rate ) ;
        Mout_7_bit ( (env->env_peak & 0x02) >> 1 ) ;
        Mout_7_bit ( ((env->env_peak & 0x01) << 6) + env->drate_1 ) ;

        pt = (BYTE *) &env->drate_2 ;
        for ( j = 0 ; j < 6 ; j++ )  Mout_7_pit ( pt[j] ) ;
    }

/*------------------------------------*/
/*  END OF FUNCTION:  "out_envelope"  */
/*------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "out_voice"                                                 */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs an entire voice.                      */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "drv" : if set to 1, output to disk, else output to MIDI.   */
/*--------------------------------------------------------------------------*/

    out_voice ( drv )
    int drv ;
    {
        int count, j ;

        /* Set "glob_drv" to indicate data destination, send start of */
        /* system exclusive and output byte count high and low bytes. */
        glob_drv = drv ;
        start_sysex ( 0 ) ;
        Mout_7_bit ( 0x04 ) ;
        Mout_7_bit ( 0x48 ) ;

        /* Initialise "glob_checksum" and send SY-22/SY-35 identity. */
        glob_checksum = 0 ;
        ident_out () ;

        /* Output the two unknown bytes */
        Mout_7_bit ( 1 ) ;
        Mout_7_bit ( 0x25 ) ;

        /* Output global voice data */
        Mout_7_bit ( (curr_data.effect << 4) + curr_data.type ) ;
        for ( count = 0 ; count < 8 ; count++ )
            Mout_7_bit ( curr_data.name[count] ) ;

        Mout_7_bit ( curr_data.num_elements ) ;
        Mout_7_bit ( curr_data.pitch_bend ) ;
        Mout_7_bit ( (curr_data.at_lvl_assign << 6) +
                     (curr_data.at_pm_assign  << 5) +
                     (curr_data.at_am_assign  << 4) +
                     (curr_data.mod_pm_assign << 1) +
                      curr_data.mod_am_assign ) ;
        Mout_8_bit ( curr_data.at_bend ) ;
        Mout_7_bit ( curr_data.env_delay ) ;
        Mout_8_bit ( curr_data.attack_rate ) ;
        Mout_8_bit ( curr_data.release_rate ) ;

        /* Output information for each element */
        for ( count = 0 ; count < 4 ; count++ )
        {
            element_type *el_pt ;
            osc_type *osc_pt ;
            int cnt ;

            /* Set pointer "el_pt" to point to start of current element */
            el_pt = &curr_data.element[count] ;

            /* Output waveform... */
            if (( count == 1 ) || ( count == 3 ))
                Mout_8_bit ( el_pt->wave ) ; /* ...if FM element */
            else
                Mout_7_bit ( el_pt->wave ) ; /* ...if AWM element */

            /* Output element pitch shift */
            Mout_8_bit ( el_pt->pitch_shift ) ;

            /* Output velocity and aftertouch response...*/
            if ( el_pt->vel_response < 0 )
                /* ...for velocity responses less than 0 */
                Mout_7_bit ( ((el_pt->at_response << 4) +
                              (el_pt->vel_response + 11)) & 0x7F ) ;
            else
                /* ...for velocity responses freater than/equal to 0 */
                Mout_7_bit ( ((el_pt->at_response << 4 ) +
                               el_pt->vel_response) & 0x7F ) ;

            /* Output LFO wave, speed, delay and rate */
            Mout_7_bit ( (el_pt->LFO_wave & 0x04) >> 2 ) ;
            Mout_7_bit ( ((el_pt->LFO_wave & 0x03) << 5) + el_pt->LFO_speed ) ;
            Mout_8_bit ( el_pt->LFO_delay ) ;
            Mout_8_bit ( el_pt->LFO_rate ) ;

            if (( count == 1 ) || ( count == 3 ))
            {
                /* Output FM tremolo assignments and depth */
                Mout_7_bit ( (el_pt->AM_carrier << 5) +
                             (el_pt->AM_modulator << 4) +
                              el_pt->LFO_AM_depth ) ;

                /* Output FM vibrato assignments and depth */
                Mout_7_bit ( (el_pt->PM_carrier << 6) +
                             (el_pt->PM_modulator << 5) +
                              el_pt->LFO_PM_depth ) ;
            }
            else
            {
                /* Output AWM tremolo and vibrato depth */
                Mout_7_bit ( el_pt->LFO_AM_depth | 0x10 ) ;
                Mout_7_bit ( el_pt->LFO_PM_depth | 0x20 ) ;
            }

            /* Output pan position */
            Mout_7_bit ( el_pt->pan ) ;

            if (( count == 1 ) || ( count == 3 ))
            {
                /* Output FM feedback and set "osc_pt" to point to modulator */
                Mout_7_bit ( el_pt->feedback ) ;
                osc_pt = &el_pt->osc_1 ;
            }
            else
                /* Set "osc_pt" to point to AWM data */
                osc_pt = &el_pt->osc_2 ;

            /* Loop for FM carrier and modulator data - */
            /* will execute only once for AWM data.     */
            for ( cnt = 0 ; cnt < 2 ; cnt++ )
            {
                if (( count == 1 ) || ( count == 3 ))
                {
                    /* Output FM fixed frequency flag, operator */
                    /* waveform and operator frequency.         */
                    Mout_7_bit ( osc_pt->fixed_frequency ) ;
                    Mout_7_bit ( (osc_pt->waveform << 4) +
                                  osc_pt->frequency ) ;
                }
                else
                    /* AWM voice, so set loop counter so that */
                    /* the loop is executed only once.        */
                    cnt = 2 ;

                /* Output level, temperament and detune */
                Mout_7_bit ( osc_pt->level ) ;
                Mout_7_bit ( (osc_pt->temperament << 4) + osc_pt->detune ) ;

                /* Output level and rate scaling, followed by envelope data */
                Mout_7_bit ( (osc_pt->level_scaling & 0x08) >> 3 ) ;
                Mout_7_bit ( ((osc_pt->level_scaling & 0x07) << 4) +
                               osc_pt->rate_scaling ) ;
                out_envelope ( &osc_pt->envelope_def ) ;

                /* Set "osc_pt" to point to FM carrier */
                osc_pt = &el_pt->osc_2 ;
            }
        }  /* End of element loop */

        /* Output vector rates */
        Mout_7_bit ( curr_data.v_level_rate & 0xF ) ;
        Mout_7_bit ( curr_data.v_detune_rate & 0xF ) ;

        /* Output level vector */
        for ( count = 0 ; count < 50 ; count++ )
        {
            Mout_8_bit ( curr_data.level_step_length[count] ) ;
            Mout_7_bit ( curr_data.level_X[count] ) ;
            Mout_7_bit ( curr_data.level_Y[count] ) ;
        }

        /* Output detune vector */
        for ( count = 0 ; count < 50 ; count++ )
        {
            Mout_8_bit ( curr_data.detune_step_length[count] ) ;
            Mout_7_bit ( curr_data.detune_X[count] ) ;
            Mout_7_bit ( curr_data.detune_Y[count] ) ;
        }

        /* Output null byte, followed by voice checksum. Note that the  */
        /* voice checksum can safely be left at zero when dealing with  */
        /* single voices - the synth will fill it in automatically when */
        /* the voice is saved. If dealing with multi-dumps, however,    */
        /* the voice checksum *must* be calculated.                     */
        Mout_7_bit ( 0 ) ;
        Mout_8_bit ( 0 ) ;

        /* Output the system exclusive cheksum and end of exclusive byte */
        Mout_7_bit ( -glob_checksum & 0x7F ) ;
        Mout_7_bit ( 0xF7 ) ;
    }

/*---------------------------------*/
/*  END OF FUNCTION:  "out_voice"  */
/*---------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTIONS:  "to_8", "to_8_t"                                            */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    These functions converts a MIDI eight-bit byte (i.e. a byte */
/*              ABCDEFGH split as 0000000A, 0BCDEFGH) into a normal byte.   */
/*              Function "to_8" uses "rx_data" as its source, and function  */
/*              "to_8_t" uses "tmp_pt" as its source.                       */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "pos" : offset of bytes from address in "rx_data"/"tmp_pt". */
/*--------------------------------------------------------------------------*/

    BYTE to_8 ( pos )
    int pos ;
    {
        return ((rx_data[pos] << 7) + rx_data[pos+1]) ;
    }

    BYTE to_8_t ( pos )
    int pos ;
    {
        return ((tmp_pt[pos] << 7) + tmp_pt[pos+1]) ;
    }

/*---------------------------------------*/
/*  END OF FUNCTIONS:  "to_8", "to_8_t"  */
/*---------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "get_envelope"                                              */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function extracts raw envelope data and places it in   */
/*              an envelope structure.                                      */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "envelope" : structure to hold processed envelope data.     */
/*              "pt"       : pointer to raw envelope data.                  */
/*--------------------------------------------------------------------------*/

    get_envelope ( envelope, pt )
    env_type *envelope ;
    BYTE *pt ;
    {
        int pos = 0 ;

        envelope->delay_switch  = pt[pos++] ;
        envelope->attack_rate   = pt[pos++] ;
        envelope->env_peak      = (pt[pos++] << 1) +
                                  ((pt[pos] >> 6) & 1 ) ;
        envelope->drate_1       = pt[pos++] & 0x3F ;
        envelope->drate_2       = pt[pos++] ;
        envelope->release_rate  = pt[pos++] ;
        envelope->initial_level = pt[pos++] ;
        envelope->attack_level  = pt[pos++] ;
        envelope->dlevel_1      = pt[pos++] ;
        envelope->dlevel_2      = pt[pos] ;
    }

/*------------------------------------*/
/*  END OF FUNCTION:  "get_envelope"  */
/*------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "save_voice"                                                */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function prompts the operator for a file name using    */
/*              the GEM file selector. If a valid filename is entered, then */
/*              the voice is saved to disk using function "out_voice".      */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    save_voice ()
    {
        int i, j, k ;
        char f_name[13], tf_name[100], temp ;
        int ret_val ;
        FILE *fpt ;

        /* Convert voice name to filename. Change all lower-cases */
        /* to upper-cases, and non-alphanumerics to underscores.  */
        i = 0 ;
        while (( curr_data.name[i] != 0 ) && ( i < 8 ))
        {
            temp = curr_data.name[i] ;
            if (( temp >= 'a' ) && ( temp <= 'z' ))
                f_name[i] = temp - 32 ;
            else if ( (( temp >= '0' ) && ( temp <= '9' )) ||
                      (( temp >= 'A' ) && ( temp <= 'Z' )) )
                f_name[i] = temp ;
            else
                f_name[i] = '_' ;

            i++ ;
        }

        /* Ensure that "f_name" is null terminated */
        f_name[i] = 0 ;

        /* Call file selector */
        fsel_input ( curr_path, f_name, &ret_val ) ;

        /* If OKAY button hit... */
        if ( ret_val == 1 )
        {
            /* Add .BNK extension if necessary */
            i = 0 ;
            j = 0 ;
            k = strlen ( f_name ) ;
            while ( i < k )
            {
                if ( f_name[i] == '.' )  j = i ;
                i++ ;
            }

            if ( j == 0 )
                strcat ( f_name, ".BNK" ) ;
            else
            {
                f_name[i+1] = 'B' ;
                f_name[i+2] = 'N' ;
                f_name[i+3] = 'K' ;
                f_name[i+4] = 0 ;
            }

            /* Construct complete filename, including path */
            j = strlen ( curr_path ) ;
            while ( curr_path[j-1] != '\\' )  j-- ;
            k = 1 ;
            strcpy ( tf_name, curr_path ) ;
            tf_name[j] = f_name[0] ;
            j++ ;
            while ( f_name[k-1] != 0 )
            {
                tf_name[j] = f_name[k] ;
                j++ ; k++ ;
            }

            /* Open binary file for writing */
            fpt = fopen ( tf_name, "wb" ) ;

            if ( fpt != NULL )
            {
                /* Output initial bytes to make file  */
                /* readable with Steinberg Satellite. */
                fputc ( 0x46, fpt ) ;
                fputc ( 0x52, fpt ) ;
                fputc ( 0x02, fpt ) ;
                fputc ( 0x7E, fpt ) ;

                /* Output voice and close file */
                out_voice ( 1 ) ;
                fclose ( fpt ) ;
            }
        }
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "save_voice"  */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "in_voice"                                                  */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function receives a voice and processes it into global */
/*              structure "curr_data".                                      */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "drive" : if set to 1, output to disk, else output to MIDI. */
/*--------------------------------------------------------------------------*/

    in_voice ( drive )
    int drive ;
    {
        int i ;
        char f_name[13], tf_name[100] ;
        int ret_val ;

        i = 1 ;
        rx_data[0] = 0 ;

        /* Check "drive" parameter */
        if ( drive == 0 )
        {
            /* Send voice request */
            start_sysex ( 0x20 ) ;
            ident_out () ;
            Mout_7_bit ( 0xF7 ) ;

            /* Wait until system exclusive data is received -   */
            /* note that MIDI channel is ignored, it just loops */
            /* until a start of exclusive byte is received.     */
            while ( rx_data[0] != 0xF0 )  rx_data[0] = Bconin ( 3 ) ;

            /* Read data until end of exclusive byte */
            /* is received or the buffer is full.    */
            while (( rx_data[i-1] != 0xF7 ) && ( i < 0x270 ))
            {
                rx_data[i] = Bconin ( 3 ) ;
                i++ ;
            }
        }
        else
        {
            int j, k ;

            /* Effective set file name to empty */
            f_name[0] = 0 ;

            /* Call file selector */
            fsel_input ( curr_path, f_name, &ret_val ) ;

            /* If OKAY button hit... */
            if ( ret_val == 1 )
            {
                /* Construct complete filename, including path */
                j = strlen ( curr_path ) ;
                while ( curr_path[j-1] != '\\' )  j-- ;
                k = 1 ;
                strcpy ( tf_name, curr_path ) ;
                tf_name[j] = f_name[0] ;
                j++ ;
                while ( f_name[k-1] != 0 )
                {
                    tf_name[j] = f_name[k] ;
                    j++ ; k++ ;
                }

                /* Open binary file for reading */
                fpt = fopen ( tf_name, "rb" ) ;

                /* Check that file was opened succesfully */
                if ( fpt != NULL )
                {
                    /* Search first 40 bytes until a start */
                    /* of exclusive byte is found.         */
                    j = 0 ;
                    while (( rx_data[0] != 0xF0 ) && ( j < 40 ))
                        rx_data[0] = fgetc ( fpt ) ;

                    /* Read data until an end of exclusive  */
                    /* byte is found or the buffer is full. */
                    while (( rx_data[i-1] != 0xF7 ) && ( i < 0x270 ))
                    {
                        rx_data[i] = fgetc ( fpt ) ;
                        i++ ;
                    }
                    fclose ( fpt ) ;
                }
            }
        }

        /* Check that last byte read is end of exclusive. This will      */
        /* effectively discard data where the input buffer has over-run. */
        if ( rx_data[i-1] == 0xF7 )
        {
            int count, j ;

            /* Extract effects data */
            curr_data.effect = (rx_data[18] >> 4) & 7 ;
            curr_data.type   = rx_data[18] & 0xF ;

            /* Extract voice name */
            for ( count = 0 ; count < 8 ; count++ )
                curr_data.name[count] = rx_data[19+count] ;
            curr_data.name[8] = 0 ;

            /* Extract number of elements and pitch bend range */
            curr_data.num_elements = rx_data[27] & 1 ;
            curr_data.pitch_bend   = rx_data[28] & 0x1F ;

            /* Extract aftertouch and mod wheel assignments */
            curr_data.at_lvl_assign = (rx_data[29] >> 6) & 1 ;
            curr_data.at_pm_assign  = (rx_data[29] >> 5) & 1 ;
            curr_data.at_am_assign  = (rx_data[29] >> 4) & 1 ;
            curr_data.mod_pm_assign = (rx_data[29] >> 1) & 1 ;
            curr_data.mod_am_assign =  rx_data[29] & 1 ;
            curr_data.at_bend = to_8 ( 30 ) ;

            /* Extract common envelope data */
            curr_data.env_delay = rx_data[32] ;
            curr_data.attack_rate = to_8 ( 33 ) ;
            curr_data.release_rate = to_8 ( 35 ) ;

            /* Loop for each element */
            for ( count = 0 ; count < 4 ; count++ )
            {
                int offset ;
                element_type *el_pt ;
                osc_type *osc_pt ;

                /* Set "tmp_pt" to point to start of */
                /* raw data of the current element.  */
                switch ( count )
                {
                    case 0 :  tmp_pt = &rx_data[37] ;  break ;
                    case 1 :  tmp_pt = &rx_data[64] ;  break ;
                    case 2 :  tmp_pt = &rx_data[111] ; break ;
                    case 3 :  tmp_pt = &rx_data[138] ; break ;
                }

                /* Set "el_pt" to point to destination of element data */
                el_pt = &curr_data.element[count] ;

                if (( count == 1 ) || ( count == 3 ))
                    /* Extract FM wave number */
                    el_pt->wave = to_8_t ( 0 ) ;
                else
                {
                    /* Extract AWM wave number */
                    el_pt->wave = tmp_pt[0] ;

                    /* Decrement pointer "tmp_pt" to allow for the */
                    /* AWM wave number only being a 7 bit value.   */
                    tmp_pt-- ;
                }

                /* Extract pitch shift from offsets 2 and 3 in raw data */
                el_pt->pitch_shift = to_8_t ( 2 ) ;

                /* Extract velocity and aftertouch response from offset 4 */
                el_pt->vel_response = tmp_pt[4] & 0xF ;
                el_pt->at_response = tmp_pt[4] >> 4 ;

                /* If velocity response greater than five, then subtract */
                /* eleven to convert it to correct negative number.      */
                if ( el_pt->vel_response > 5 )
                    el_pt->vel_response -= 11 ;

                /* Extract LFO data from offsets 5 to 10 */
                el_pt->LFO_wave = (tmp_pt[5] << 2) + (tmp_pt[6] >> 5) ;
                el_pt->LFO_speed = tmp_pt[6] & 0x1F ;
                el_pt->LFO_delay = to_8_t ( 7 ) ;
                el_pt->LFO_rate  = to_8_t ( 9 ) ;

                /* Extract tremolo information from offset 11 */
                el_pt->AM_carrier = tmp_pt[11] >> 5 ;
                el_pt->AM_modulator = (tmp_pt[11] >> 4) & 1 ;
                el_pt->LFO_AM_depth = tmp_pt[11] & 0xF ;

                /* Extract vibrato information from offset 12 */
                el_pt->PM_carrier = tmp_pt[12] >> 6 ;
                el_pt->PM_modulator = (tmp_pt[12] >> 5) & 1 ;
                el_pt->LFO_PM_depth = tmp_pt[12] & 0x1F ;

                /* Extract envelope number and pan position from offset 13 */
                el_pt->envelope = tmp_pt[13] >> 4 ;
                el_pt->pan = tmp_pt[13] & 0xF ;

                /* Check current element being extracted...*/
                if (( count == 1 ) || ( count == 3 ))
                {
                    /* ...FM element*/

                    /* Extract FM feedback from offset 14 */
                    el_pt->feedback = tmp_pt[14] ;

                    /* Set "pt2" to point to start of modulator data */
                    pt2 = &tmp_pt[15] ;

                    /* Set "osc_pt" to point to destination */
                    /* for processed modulator data.        */
                    osc_pt = &el_pt->osc_1 ;

                    /* Loop twice - first for modulator, second for carrier */
                    for ( j = 0 ; j < 2 ; j++ )
                    {
                        /* Extract operator data */
                        osc_pt->fixed_frequency = pt2[0] ;
                        osc_pt->waveform = pt2[1] >> 4 ;
                        osc_pt->frequency = pt2[1] & 0xF ;
                        osc_pt->level = pt2[2] ;
                        osc_pt->temperament = pt2[3] >> 4 ;
                        osc_pt->detune = pt2[3] & 0xF ;
                        osc_pt->level_scaling = (pt2[4] << 3) + (pt2[5] >> 4) ;
                        osc_pt->rate_scaling = pt2[5] & 0xF ;

                        /* Extract operator envelope */
                        get_envelope ( &osc_pt->envelope_def, &pt2[6] ) ;

                        /* Set "pt2" to point to start of carrier data */
                        pt2 = &tmp_pt[31] ;

                        /* Set "osc_pt" to point to destination */
                        /* for processed carrier data.          */
                        osc_pt = &el_pt->osc_2 ;
                    }
                }
                else
                {
                    /* ...AWM element */

                    /* Extract element level from offset 14 */
                    el_pt->osc_2.level       = tmp_pt[14] ;

                    /* Extract temperament and detune from offset 15 */
                    el_pt->osc_2.temperament = tmp_pt[15] >> 4 ;
                    el_pt->osc_2.detune      = tmp_pt[15] & 0xF ;

                    /* Extract level scaling from offset 16 */
                    el_pt->osc_2.level_scaling = (tmp_pt[16] << 3) + 
                                                 (tmp_pt[16] >> 4) ;

                    /* Extract rate_scaling from offset 17 */
                    el_pt->osc_2.rate_scaling  = tmp_pt[17] & 0xF ;

                    /* Extract envelope starting at offset 18 */
                    get_envelope ( &el_pt->osc_2.envelope_def, &tmp_pt[18] ) ;
                }
            }

            /* Extract vector level and detune rates */
            curr_data.v_level_rate = rx_data[185] ;
            if ( curr_data.v_level_rate == 0 )  curr_data.v_level_rate = 16 ;
            curr_data.v_detune_rate = rx_data[186] ;
            if ( curr_data.v_detune_rate == 0 ) curr_data.v_detune_rate = 16 ;

            /* Extract level vector */
            for ( count = 0 ; count < 50 ; count++ )
            {
                curr_data.level_step_length[count] = to_8 ( 187 + (count * 4) ) ;
                curr_data.level_X[count] = rx_data [ 189 + (count * 4) ] ;
                curr_data.level_Y[count] = rx_data [ 190 + (count * 4) ] ;
            }

            /* Extract detune vector */
            for ( count = 0 ; count < 50 ; count++ )
            {
                curr_data.detune_step_length[count] = to_8 ( 387 + (count * 4) ) ;
                curr_data.detune_X[count] = rx_data [ 389 + (count * 4) ] ;
                curr_data.detune_Y[count] = rx_data [ 390 + (count * 4) ] ;
            }

            /* Copy "curr_data" (as loaded above) into "last_data" */
            pt = (BYTE *) &curr_data ;
            pt2 = (BYTE *) &last_data ;
            for ( count = 0 ; count < 540 ; count++ ) pt2[count] = pt[count] ;
        }
    }

/*--------------------------------*/
/*  END OF FUNCTION:  "in_voice"  */
/*--------------------------------*/
