/*---------------------------------------------------------------------------*/
/* File:     SY_VDI.C                                                        */
/* Author:   R. P. Hanson                                                    */
/* Copyright:R. P. Hanson 1996                                               */
/*           Lothar Bittner is now the authorised custodian of the Atari ST  */
/*           version of this program.                                        */
/*---------------------------------------------------------------------------*/
/* Contents: Functions to deal with displaying of data.                      */
/*---------------------------------------------------------------------------*/
/* Functions:"text_out"       displays a text string.                        */
/*           "erase_block"    erases an area of the display.                 */
/*           "vel_out"        graphically displays the velocity response.    */
/*           "AT_out"         graphically displays the aftertouch response.  */
/*           "LFO_wv_out"     graphically displays the LFO waveform.         */
/*           "level_out"      graphically displays the volume of an element. */
/*           "pan_out"        graphically displays the pan of an element.    */
/*           "ls_out"         graphically displays the level scaling.        */
/*           "rs_out"         graphically displays the rate scaling.         */
/*           "draw_env"       draws the envelope of an element.              */
/*           "init_app"       called to initialise the VDI.                  */
/*           "close_app"      called to close the VDI before exiting.        */
/*           "draw_box"       draws a box.                                   */
/*           "rounded_box"    draws a rounded box.                           */
/*           "mark_point"     draws a single point.                          */
/*           "draw_line"      draws a straight line.                         */
/*           "draw_main_screen" draws the static screen information.         */
/*           "clear_work_area"  clears the edit area (bottom of display).    */
/*           "draw_vgraph"    draws the vector information.                  */
/*           "init_but_flag"  initialises the state of a button.             */
/*           "check_but_stat" checks whether a button is pressed.            */
/*           "draw_common"    draws static text for editing elements.   .    */
/*           "show_env"       draws static text for editing envelopes.       */
/*           "draw_awm_element" draws static text for editing AWM elements.  */
/*           "draw_fm_element"  draws static text for editing FM elements.   */
/*           "draw_vector"    draws static text for vector display.          */
/*           "draw_global"    draws static text for editing voice common.    */
/*---------------------------------------------------------------------------*/
/* Globals:  "el_a_butt", "el_b_butt", "el_c_butt", "el_d_butt"              */
/*                            co-ordinates for element buttons A to D        */
/*           "vect_butt", "glob_butt", "load_butt", "save_butt"              */
/*                            co-ordinates for buttons for vector, voice     */
/*                            common, load and save.                         */
/*           "get_butt", "quit_butt"                                         */
/*                            co-ordinates for buttons get and exit.         */
/*           "el_a_y1", "el_a_y2", "el_a_y3"                                 */
/*                            y values for lines in element A area.          */
/*           "el_b_y1", "el_b_y2", "el_b_y3"                                 */
/*                            y values for lines in element B area.          */
/*           "el_c_y1", "el_c_y2", "el_c_y3"                                 */
/*                            y values for lines in element C area.          */
/*           "el_d_y1", "el_d_y2", "el_d_y3"                                 */
/*                            y values for lines in element D area.          */
/*           "glob_y1", "glob_y2", "glob_y3"                                 */
/*                            y values for lines in voice common area.       */
/*           "vect_y1", "vect_y2", "vect_y3"                                 */
/*                            y values for lines in vector area.             */
/*           "curr_edit"      current edit mode, 0 to 5.                     */
/*           "y_vals"         list of y values for text lines in edit area.  */
/*---------------------------------------------------------------------------*/


/*----------*/
/* Includes */
/*----------*/
    #include <gemfast.h>
    #include "sy_vdi.h"
    #include "types.h"

/*-----------------------*/
/* Define button borders */
/*-----------------------*/
    rect_type   el_a_butt = {   A_BUT_LEFT,     A_BUT_TOP,
                                A_BUT_RIGHT,    A_BUT_BOT } ;
    rect_type   el_b_butt = {   B_BUT_LEFT,     B_BUT_TOP,
                                B_BUT_RIGHT,    B_BUT_BOT } ;
    rect_type   el_c_butt = {   C_BUT_LEFT,     C_BUT_TOP,
                                C_BUT_RIGHT,    C_BUT_BOT } ;
    rect_type   el_d_butt = {   D_BUT_LEFT,     D_BUT_TOP,
                                D_BUT_RIGHT,    D_BUT_BOT } ;
    rect_type   vect_butt = {   VECT_BUT_LEFT,  VECT_BUT_TOP,
                                VECT_BUT_RIGHT, VECT_BUT_BOT } ;
    rect_type   glob_butt = {   GLOB_BUT_LEFT,  GLOB_BUT_TOP,
                                GLOB_BUT_RIGHT, GLOB_BUT_BOT } ;
    rect_type   load_butt = {   LD_BUT_LEFT,    LD_BUT_TOP,
                                LD_BUT_RIGHT,   LD_BUT_BOT } ;
    rect_type   save_butt = {   SV_BUT_LEFT,    SV_BUT_TOP,
                                SV_BUT_RIGHT,   SV_BUT_BOT } ;
    rect_type   get_butt  = {   GET_BUT_LEFT,   GET_BUT_TOP,
                                GET_BUT_RIGHT,  GET_BUT_BOT } ;
    rect_type   quit_butt = {   QT_BUT_LEFT,    QT_BUT_TOP,
                                QT_BUT_RIGHT,   QT_BUT_BOT } ;

/*---------------------------------------*/
/* Define variables required by the VDI. */
/*---------------------------------------*/
    int contrl[12] ;
    int intin[128] ;
    int ptsin[128] ;
    int intout[128] ;
    int ptsout[128] ;
    int handle ;
    int phys_handle ;
    int work_in[12] ;
    int work_out[57] ;
    int pxarray[30] ;
    int ch_w ;
    int ch_h ;
    int ce_w ;
    int ce_h ;
    int bo_w ;
    int bo_h ;

/*---------------------------------*/
/* Variables defining screen areas */
/*---------------------------------*/
    int el_a_y1 ;    int el_a_y2 ;    int el_a_y3 ;
    int el_b_y1 ;    int el_b_y2 ;    int el_b_y3 ;
    int el_c_y1 ;    int el_c_y2 ;    int el_c_y3 ;
    int el_d_y1 ;    int el_d_y2 ;    int el_d_y3 ;
    int glob_y1 ;    int glob_y2 ;    int glob_y3 ;
    int vect_y1 ;    int vect_y2 ;    int vect_y3 ;
    int y_vals[16] = { 232, 240, 248, 256, 264, 272, 280, 288, 296, 304, 312,
                       320, 328, 336, 344, 352 } ;
    int curr_edit ;

/*--------------------*/
/* External variables */
/*--------------------*/
    extern int mx, my, mbut, kstat ;

/*---------------------------------------*/
/* Defines for turning cursor off and on */
/*---------------------------------------*/
    #define m_point_off         v_hide_c ( handle )
    #define m_point_on          v_show_c ( handle, 0 )

/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "text_out"                                                  */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs a text string at the specified screen */
/*              location.                                                   */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x", "y" : starting co-ordinate.                            */
/*              "txt"    : string to display.                               */
/*--------------------------------------------------------------------------*/

    text_out ( x, y, txt )
    int x, y ;
    char *txt ;
    {
        v_gtext ( handle, x, y, txt ) ;
    }

/*--------------------------------*/
/*  END OF FUNCTION:  "text_out"  */
/*--------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "erase_block"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function blanks out the specified rectangle.           */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x",  "y"  : the bottom left co-ordinate.                   */
/*              "xt", "yt" : the width and height to be erased.             */
/*--------------------------------------------------------------------------*/

    erase_block ( x, y, xt, yt )
    int x, y, xt, yt ;
    {
        /* Set colour to white, replace mode, perimeter on, solid interior */
        vswr_mode ( handle, 1 ) ;
        vsf_color ( handle, 0 ) ;
        vsf_perimeter ( handle, 1 ) ;
        vsf_interior ( handle, 1 ) ;

        /* Define X points */
        pxarray[0] = pxarray[6] = pxarray[8]= x ;
        pxarray[2] = pxarray[4] = x + xt ;

        /* Define Y points */
        pxarray[1] = pxarray[3] = pxarray[9] = y - yt ;
        pxarray[5] = pxarray[7] = y ;

        /* Fill area */
        v_fillarea ( handle, 5, pxarray ) ;

        /* Set colour to black */
        vsf_color ( handle, 1 ) ;
    }

/*-----------------------------------*/
/*  END OF FUNCTION:  "erase_block"  */
/*-----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "vel_out"                                                   */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function graphically displays the velocity response.   */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x", "y" : starting co-ordinate.                            */
/*              "vel"    : velocity curve required.                         */
/*              "wi"     : divisor used to determine width of curve.        */
/*--------------------------------------------------------------------------*/

    vel_out ( x, y, vel, wi )
    int x, y ;
    char vel ;
    {
        int pnts ;
        erase_block ( x, y, wi, 7 ) ;
        pxarray[0] = x ;
        pnts = 2 ;

        switch ( vel )
        {
            case -5 :                               pxarray[1] = y - 7 ;
                        pxarray[2] = x + (wi / 2) ; pxarray[3] = y - 7 ;
                        pxarray[4] = x + (wi / 2) ; pxarray[5] = y ;
                        pxarray[6] = x + wi ;       pxarray[7] = y ;
                        pnts = 4 ;  break ;
            case -4 :                               pxarray[1] = y - 7 ;
                        pxarray[2] = x + (wi / 2) ; pxarray[3] = y - 7 ;
                        pxarray[4] = x + wi ;       pxarray[5] = y ;
                        pnts = 3 ;  break ;
            case -3 :                               pxarray[1] = y - 7 ;
                        pxarray[2] = x + wi ;       pxarray[3] = y ;
                        break ;
            case -2 :                               pxarray[1] = y - 7 ;
                        pxarray[2] = x + wi ;       pxarray[3] = y - 2 ;
                        break ;
            case -1 :                               pxarray[1] = y - 7 ;
                        pxarray[2] = x + wi ;       pxarray[3] = y - 5 ;
                        break ;
            case 0 :                                pxarray[1] = y - 4 ;
                        pxarray[2] = x + wi ;       pxarray[3] = y - 4 ;
                        break ;
            case 1 :                                pxarray[1] = y - 5 ;
                        pxarray[2] = x + wi ;       pxarray[3] = y - 7 ;
                        break ;
            case 2 :                                pxarray[1] = y - 2 ;
                        pxarray[2] = x + wi ;       pxarray[3] = y - 7 ;
                        break ;
            case 3 :                                pxarray[1] = y ;
                        pxarray[2] = x + wi ;       pxarray[3] = y - 7 ;
                        break ;
            case 4 :                                pxarray[1] = y ;
                        pxarray[2] = x + (wi / 2) ; pxarray[3] = y ;
                        pxarray[4] = x + wi ;       pxarray[5] = y - 7 ;
                        pnts = 3 ;  break ;
            case 5 :                                pxarray[1] = y ;
                        pxarray[2] = x + (wi / 2) ; pxarray[3] = y ;
                        pxarray[4] = x + (wi / 2) ; pxarray[5] = y - 7 ;
                        pxarray[6] = x + wi ;       pxarray[7] = y - 7 ;
                        pnts = 4 ;
        }
        v_pline ( handle, pnts, pxarray ) ;
    }

/*-------------------------------*/
/*  END OF FUNCTION:  "vel_out"  */
/*-------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "AT_out"                                                    */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs the aftertouch response.              */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "AT" : Aftertouch response.                                 */
/*--------------------------------------------------------------------------*/

    int AT_lines[7][4] =  { { 154, 242,   198, 249 },
                            { 154, 242,   198, 247 },
                            { 154, 242,   198, 244 },
                            { 154, 245,   198, 245 },
                            { 154, 244,   198, 242 },
                            { 154, 247,   198, 242 },
                            { 154, 249,   198, 242 } } ;

    AT_out ( AT )
    char AT ;
    {
        erase_block ( 154, 249, 44, 7 ) ;
        v_pline ( handle, 2, AT_lines[AT+3] ) ;
    }

/*------------------------------*/
/*  END OF FUNCTION:  "AT_out"  */
/*------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "LFO_wv_out"                                                */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs the LFO waveform.                     */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "wave" : waveform to be displayed.                          */
/*--------------------------------------------------------------------------*/

    int LFO_wvs[5][33] = {{ 7, 154, 257,   154, 250,   168, 257,   168, 250,
                               182, 257,   182, 250,   196, 257,     0, 0,
                                 0, 0,       0, 0,       0, 0,       0, 0,
                                 0, 0,       0, 0,       0, 0,       0, 0 },
                          { 5, 156, 254,   162, 250,   174, 257,   186, 250,
                               192, 254,     0, 0,       0, 0,       0, 0,
                                 0, 0,       0, 0,       0, 0,       0, 0,
                                 0, 0,       0, 0,       0, 0,       0, 0 },
                          { 13,154, 257,   163, 257,   163, 250,   170, 250,
                               170, 257,   177, 257,   177, 250,   184, 250,
                               184, 257,   191, 257,   191, 250,   198, 250,
                               198, 257,     0, 0,       0, 0,       0, 0 },
                          { 16,154, 250,   157, 255,   160, 252,   163, 257,
                               166, 255,   169, 257,   172, 256,   175, 250,
                               178, 252,   181, 253,   184, 257,   187, 252,
                               190, 257,   193, 255,   196, 250,   198, 252 },
                          { 7, 154, 257,   168, 250,   168, 257,   182, 250,
                               182, 257,   196, 250,   196, 257,     0, 0,
                                 0, 0,       0, 0,       0, 0,       0, 0,
                                 0, 0,       0, 0,       0, 0,       0, 0 } } ;

    LFO_wv_out ( wave )
    char wave ;
    {
        erase_block ( 154, 257, 48, 7 ) ;
        v_pline ( handle, LFO_wvs[wave][0], &LFO_wvs[wave][1] ) ;
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "LFO_wv_out"  */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "level_out"                                                 */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs a bar-graph indicating the level.     */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "y"     : y position to display at.                         */
/*              "level" : level.                                            */
/*--------------------------------------------------------------------------*/

    level_out ( y, level )
    int y ;
    char level ;
    {
        int hlev, neg_lev ;

        hlev = (int) level / 2 ;
        neg_lev = 63 - hlev ;
        vsf_interior ( handle, 1 ) ;
        pxarray[1] = y - 1 ;
        pxarray[3] = y - 6 ;
        if ( hlev > 0 )
        {
            pxarray[0] = 61 ;           pxarray[1] = y - 1 ;
            pxarray[2] = 61 + hlev ;    pxarray[3] = y - 6 ;
            v_bar ( handle, pxarray ) ;
        }
        if ( neg_lev > 0 )
        {
            pxarray[0] = 124 - neg_lev ; pxarray[1] = y - 1 ;
            pxarray[2] = 124 ;           pxarray[3] = y - 6 ;
            vsf_color ( handle, 0 ) ;
            v_bar ( handle, pxarray ) ;
            vsf_color ( handle, 1 ) ;
        }
    }

/*---------------------------------*/
/*  END OF FUNCTION:  "level_out"  */
/*---------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "pan_out"                                                   */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs a box graphically representing pan.   */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "y"    : y position to display at.                          */
/*              "posn" : pan position.                                      */
/*--------------------------------------------------------------------------*/

    pan_out ( y, posn )
    int y ;
    unsigned char posn ;
    {
        unsigned char posneg ;

        posneg = 4 - posn ;

        vsf_color ( handle, 0 ) ;
        pxarray[0] = 61 ;   pxarray[1] = y - 1 ;
        pxarray[2] = 124 ;  pxarray[3] = y - 6 ;
        v_bar ( handle, pxarray ) ;
        vsf_color ( handle, 1 ) ;

        pxarray[1] = pxarray[7] = pxarray[9] = y - 2 ;
        pxarray[3] = pxarray[5] = y - 5 ;

        pxarray[0] = pxarray[2] = pxarray[8] = ( posneg * 12 ) + 67 ;
        pxarray[4] = pxarray[6] = pxarray[0] + 4 ;

        v_pline ( handle, 5, pxarray ) ;
    }

/*-------------------------------*/
/*  END OF FUNCTION:  "pan_out"  */
/*-------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "ls_out"                                                    */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs the level scaling graph.              */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x"   : x position to display at.                           */
/*              "lvl" : level scaling position.                             */
/*--------------------------------------------------------------------------*/

    int ls_wave[16][9] = {  { 2,  0, 252,  32, 252,   0, 0,     0, 0   },
                            { 3,  0, 249,  27, 249,  32, 252,   0, 0   },
                            { 3,  0, 249,  21, 249,  32, 253,   0, 0   },
                            { 3,  0, 249,  21, 249,  32, 256,   0, 0   },
                            { 3,  0, 249,  17, 249,  32, 256,   0, 0   },
                            { 3,  0, 249,  17, 249,  32, 253,   0, 0   },
                            { 2,  0, 249,  32, 253,   0, 0,     0, 0   },
                            { 2,  0, 249,  32, 256,   0, 0,     0, 0   },
                            { 4,  0, 249,  10, 249,  21, 253,  32, 256 },
                            { 3,  0, 256,  10, 249,  32, 249,   0, 0   },
                            { 2,  0, 256,  32, 249,   0, 0,     0, 0   },
                            { 4,  0, 253,  10, 249,  21, 249,  32, 253 },
                            { 4,  0, 256,  10, 256,  21, 249,  32, 249 },
                            { 4,  0, 249,  10, 249,  21, 256,  32, 256 },
                            { 4,  0, 256,  16, 256,  16, 249,  32, 249 },
                            { 4,  0, 249,  15, 249,  15, 256,  32, 256 } } ;

    ls_out ( x, lvl )
    int x ;
    unsigned char lvl ;
    {
        int *ptr, list[10], i ;

        ptr = ls_wave[lvl] ;
        for ( i = 0 ; i <= 8 ; i++ )    list[i] = ptr[i] ;
        for ( i = 1 ; i <= 8 ; i += 2 ) list[i] += x ;
        erase_block ( x, 256, 33, 7 ) ;
        v_pline ( handle, list[0], &list[1] ) ;
    }

/*------------------------------*/
/*  END OF FUNCTION:  "ls_out"  */
/*------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "rs_out"                                                    */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function outputs the rate scaling.                     */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x"    : x position to display at.                          */
/*              "rate" : rate scaling.                                      */
/*--------------------------------------------------------------------------*/

    int rs_wave[8][7] = {   { 2,  0, 252,  32, 252,   0, 0   },
                            { 3,  0, 249,  27, 249,  32, 252 },
                            { 3,  0, 249,  17, 249,  32, 253 },
                            { 2,  0, 249,  32, 253,   0, 0   },
                            { 2,  0, 249,  32, 256,   0, 0   },
                            { 3,  0, 249,  17, 249,  32, 256 },
                            { 2,  0, 253,  32, 249,   0, 0   },
                            { 3,  0, 256,  10, 249,  32, 249 } } ;

    rs_out ( x, rate )
    int x ;
    unsigned char rate ;
    {
        int *ptr, list[10], i ;

        ptr = rs_wave[rate] ;
        for ( i = 0 ; i <= 8 ; i++ )    list[i] = ptr[i] ;
        for ( i = 1 ; i <= 8 ; i += 2 )
        {
            list[i] += x ;
            list[i+1] += 8 ;
        }
        erase_block ( x, 264, 33, 7 ) ;
        v_pline ( handle, list[0], &list[1] ) ;
    }

/*------------------------------*/
/*  END OF FUNCTION:  "rs_out"  */
/*------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_env"                                                  */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the input envelope. If the envelope is  */
/*              for an FM modulator, the graph is a broken line.            */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "y"    : y position to display at.                          */
/*              "env"  : envelope structure.                                */
/*              "modw" : if 1, then output modulator graph.                 */
/*--------------------------------------------------------------------------*/

    draw_env ( y, env, modw )
    int y ;
    env_type env ;
    unsigned char modw ;
    {
        int i, j ;
        char tmp ;

        i = 2 ;
        j = 5 ;
        if ( modw == 0 )  erase_block ( 208, y+1, 390, 26 ) ;

        /* Initial level */
        pxarray[0] = 210 ;
        pxarray[1] = y - (( 23 * ( 127 - env.initial_level )) / 127 ) ;

        /* Check whether envelope delay is selected. */
        if ( env.delay_switch == 1 )
        {
            j++ ;
            pxarray[i++] = pxarray[0] + ( 127 - curr_data.env_delay ) ;
            pxarray[i++] = pxarray[1] ;
        }

        /* Attack rate and level */
        tmp = 63 - env.attack_rate ;
        if ( modw == 0 )  tmp -= curr_data.attack_rate ;
        if ( tmp < 0 )  tmp = 0 ;
        pxarray[i] = pxarray[i-2] + tmp ;  i++ ;
        pxarray[i++] = y - (( 23 * ( 127 - env.attack_level )) / 127 ) ;

        /* Decay 1 rate and level */
        tmp = 63 - env.drate_1 ;
        pxarray[i] = pxarray[i-2] + tmp ;  i++ ;
        pxarray[i++] = y - (( 23 * ( 127 - env.dlevel_1 )) / 127 ) ;

        /* Decay 2 rate and level */
        tmp = 63 - env.drate_2 ;
        pxarray[i] = pxarray[i-2] + tmp ;  i++ ;
        pxarray[i++] = y - (( 23 * ( 127 - env.dlevel_2 )) / 127 ) ;

        /* Release rate */
        tmp = 63 - env.release_rate ;
        if ( modw == 0 )  tmp -= curr_data.release_rate ;
        if ( tmp < 0 )  tmp = 0 ;
        pxarray[i] = pxarray[i-2] + tmp ;  i++ ;
        pxarray[i] = y ;

        /* If modulator, select broken line, else select thick solid line */
        if ( modw == 1 )  vsl_type ( handle, 3 ) ;
        else              vsl_width ( handle, 3 ) ;

        /* Output line */
        v_pline ( handle, j, pxarray ) ;

        /* Reset thickness and solidity */
        vsl_type ( handle, 1 ) ;
        vsl_width ( handle, 1 ) ;
    }

/*--------------------------------*/
/*  END OF FUNCTION:  "draw_env"  */
/*--------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "init_app"                                                  */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function initialises the AES and the VDI.              */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    init_app ()
    {
        int i ;

        /* Initialiase application */
        appl_init () ;
        for ( i = 0 ; i < 10 ; work_in[i++] = 1 ) ;
        work_in[10] = 0 ;
        v_opnvwk ( work_in, &handle, work_out ) ;

        /* Clear screen */
        m_point_off ;
        v_clrwk ( handle ) ;
        m_point_on ;
    }

/*--------------------------------*/
/*  END OF FUNCTION:  "init_app"  */
/*--------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "close_app"                                                 */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function closes the AES and the VDI.                   */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    close_app ()
    {
        /* CLOSE DOWN APPLICATION */
        v_clsvwk ( handle ) ;
        appl_exit () ;
    }

/*---------------------------------*/
/*  END OF FUNCTION:  "close_app"  */
/*---------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_box"                                                  */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws a box.                                  */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x1", "y1" : Bottom left co-ordinate.                       */
/*              "x2", "y2" : Bottom right co-ordinate.                      */
/*--------------------------------------------------------------------------*/

    draw_box ( x1, y1, x2, y2 )
    int x1, y1, x2, y2 ;
    {
        pxarray[0] = x1 ;   pxarray[1] = y1 ;
        pxarray[2] = x2 ;   pxarray[3] = y1 ;
        pxarray[4] = x2 ;   pxarray[5] = y2 ;
        pxarray[6] = x1 ;   pxarray[7] = y2 ;
        pxarray[8] = x1 ;   pxarray[9] = y1 ;
        v_pline ( handle, 5, pxarray ) ;
    } 

/*--------------------------------*/
/*  END OF FUNCTION:  "draw_box"  */
/*--------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "rounded_box"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws a rounded box.                          */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "params" : pointer to co-ordinate list.                     */
/*--------------------------------------------------------------------------*/

    rounded_box ( params )
    int *params ;
    {
        v_rbox ( handle, params ) ;
    }

/*-----------------------------------*/
/*  END OF FUNCTION:  "rounded_box"  */
/*-----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "mark_point"                                                */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws a single point.                         */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x", "y" : point to mark.                                   */
/*--------------------------------------------------------------------------*/

    mark_point ( x, y )
    {
        pxarray[0] = x ;  pxarray[1] = y ;
        v_pmarker ( handle, 1, pxarray ) ;
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "mark_point"  */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_line"                                                 */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws a single line.                          */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x1", "y1" : start co-ordinate.                             */
/*              "x2", "y2" : end co-ordinate.                               */
/*--------------------------------------------------------------------------*/

    draw_line ( x1, y1, x2, y2 )
    {
        pxarray[0] = x1 ;   pxarray[1] = y1 ;
        pxarray[2] = x2 ;   pxarray[3] = y2 ;
        v_pline ( handle, 2, pxarray ) ;
    }

/*---------------------------------*/
/*  END OF FUNCTION:  "draw_line"  */
/*---------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_main_screen"                                          */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the static elements of the display.     */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    draw_main_screen ()
    {
        char text[80] ;
        int  i, y1, y2, y3 ;

        /* Select replace mode and turn off cursor */
        vswr_mode ( handle, 1 ) ;
        vsf_color ( handle, 1 ) ;
        m_point_off ;

        /* Dot each corner of the display */
        mark_point ( 0, 0 ) ;
        mark_point ( 639, 0 ) ;
        mark_point ( 639, 399 ) ;
        mark_point ( 0, 399 ) ;

        /* Draw display frame */
        vsl_width ( handle, 5 ) ;
        vsl_ends ( handle, 2, 2 ) ;
        pxarray[0] = 10 ;   pxarray[1] = 3 ;
        pxarray[2] = 635 ;  pxarray[3] = 3 ;
        pxarray[4] = 635 ;  pxarray[5] = 396 ;
        pxarray[6] = 4 ;    pxarray[7] = 396 ;
        pxarray[8] = 4 ;    pxarray[9] = 10 ;
        v_pline ( handle, 5, pxarray ) ;

        /* Display title and underline */
        vst_point ( handle, 9, &ch_w, &ch_h, &ce_w, &ce_h ) ;
        vst_effects ( handle, 16 ) ;
        text_out ( 10, 14, "SY-22/35 Voice Editor V1.1" ) ;
        vsl_width ( handle, 3 ) ;
        draw_line ( 4, 18, 634, 18 ) ;

        /* Reset graphics effects */
        vsl_width ( handle, 1 ) ;
        vsl_ends ( handle, 0, 0 ) ;
        vst_effects ( handle, 0 ) ;

        /* Draw all buttons */
        rounded_box ( &el_a_butt ) ;
        text_out ( el_a_butt.left + 17, el_a_butt.bottom - 2, "A" ) ;
        rounded_box ( &el_b_butt ) ;
        text_out ( el_b_butt.left + 17, el_b_butt.bottom - 2, "B" ) ;
        rounded_box ( &el_c_butt ) ;
        text_out ( el_c_butt.left + 17, el_c_butt.bottom - 2, "C" ) ;
        rounded_box ( &el_d_butt ) ;
        text_out ( el_d_butt.left + 17, el_d_butt.bottom - 2, "D" ) ;
        rounded_box ( &glob_butt ) ;
        text_out ( glob_butt.left + 9, glob_butt.bottom - 2, "Global" ) ;
        rounded_box ( &vect_butt ) ;
        text_out ( vect_butt.left + 9, vect_butt.bottom - 2, "Vector" ) ;
        rounded_box ( &load_butt ) ;
        text_out ( load_butt.left + 17, load_butt.bottom - 2, "Load" ) ;
        rounded_box ( &save_butt ) ;
        text_out ( save_butt.left + 17, save_butt.bottom - 2, "Save" ) ;
        rounded_box ( &get_butt ) ;
        text_out ( get_butt.left + 21, get_butt.bottom - 2, "Get" ) ;
        rounded_box ( &quit_butt ) ;
        text_out ( quit_butt.left + 17, quit_butt.bottom - 2, "Quit" ) ;
        draw_line ( 4, 380, 634, 380 ) ;

        /* Draw lines between each edit area */
        strcpy ( text, "ELEMENT A" ) ;
        for ( i = 0 ; i < 6 ; i++ )
        {
            y1 = 28 + (i * 32) ;
            y2 = 38 + (i * 32) ;
            y3 = 48 + (i * 32) ;

            /* Check area being processed... */
            if ( i < 4 )
            {
                /* ...element areas */

                /* Store Y positions */
                switch ( i )
                {
                    case 0 :
                        el_a_y1 = y1 ; el_a_y2 = y2 ; el_a_y3 = y3 ;
                        break ;
                    case 1 :
                        el_b_y1 = y1 ; el_b_y2 = y2 ; el_b_y3 = y3 ;
                        break ;
                    case 2 :
                        el_c_y1 = y1 ; el_c_y2 = y2 ; el_c_y3 = y3 ;
                        break ;
                    case 3 :
                        el_d_y1 = y1 ; el_d_y2 = y2 ; el_d_y3 = y3 ;
                }

                /* Display element name... */
                text[8] = 65 + i ;
                vst_effects ( handle, 1 ) ;
                text_out ( 10, y1, text ) ;
                vst_effects ( handle, 0 ) ;

                /* ...level and pan */
                text_out ( 10, y2, "Level:" ) ;
                draw_box ( 60, y2, 125, y2 - 7 ) ;
                text_out ( 10, y3, "Pan:" ) ;
                draw_box ( 60, y3, 125, y3 - 7 ) ;
            }
            else if ( i == 4 )
            {
                /* ...voice common area */

                /* Store Y positions */
                glob_y1 = y1 ;
                glob_y2 = y2 ;
                glob_y3 = y3 ;

                /* Display title... */
                vst_effects ( handle, 1 ) ;
                text_out ( 10, y1, "GLOBAL VOICE DATA" ) ;
                vst_effects ( handle, 0 ) ;

                /* ...voice config and name... */
                text_out ( 10, y2, "Voice name:" ) ;
                text_out ( 10, y3, "Elements:" ) ;

                /* ...effects info... */
                text_out ( 180, y1, "Effect:" ) ;
                text_out ( 180, y2, "Depth:" ) ;
                text_out ( 180, y3, "Pitch bend:" ) ;

                /* ...modulation info */
                text_out ( 326, y1, "Wheel AM:" ) ;
                text_out ( 326, y2, "Wheel PM:" ) ;
                text_out ( 326, y3, "A.T. AM:" ) ;
                text_out ( 440, y1, "A.T. PM:" ) ;
                text_out ( 440, y2, "A.T. Level:" ) ;
                text_out ( 440, y3, "A.T. Pitch:" ) ;
            }
            else
            {
                /* ...vector area */

                /* Store Y positions */
                vect_y1 = y1 ;
                vect_y2 = y2 ;
                vect_y3 = y3 ;

                /* Display title... */
                vst_effects ( handle, 1 ) ;
                text_out ( 10, y1, "VECTOR" ) ;
                vst_effects ( handle, 0 ) ;

                /* ...type... */
                text_out ( 10, y2, "Level....Speed:     ms" ) ;
                text_out ( 10, y3, "Detune...Speed:     ms" ) ;

                /* ...vector items */
                text_out ( 200, y2, "Item   ...X:     Y:     Time:" ) ;
                text_out ( 200, y3, "Item   ...X:     Y:     Time:" ) ;
            }

            draw_line ( 4, 19 + ((i + 1) * 32), 634, 19 + ((i + 1) * 32) ) ;
        }
        m_point_on ;
    }

/*----------------------------------------*/
/*  END OF FUNCTION:  "draw_main_screen"  */
/*----------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "clear_work_area"                                           */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function clears the main editing area.                 */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    clear_work_area ()
    {
        erase_block ( 8, 379, 623, 165 ) ;
    }

/*--------------------------------------*/
/*  END OF FUNCTION:  "clear_work_area" */
/*--------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_vgraph"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the vector information.                 */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    draw_vgraph ()
    {
        int i, x, k ;

        m_point_off ;
        erase_block ( 100, 379, 531, 165 ) ;
        vswr_mode ( handle, 3 ) ;

        /* Draw level vector */
        i = 0 ;
        x = 100 ;
        while (( x < 630 ) && ( curr_data.level_step_length[i] != 255 ) &&
               ( i < 50 ))
        {
            if ( curr_data.level_step_length[i] < 254 )
                for ( k = 0 ; k < curr_data.level_step_length[i]+1 ; k++ )
                {
                    if ( x < 630 )
                    {
                        draw_line ( x, 250,
                                    x, 265 - (curr_data.level_X[i] / 2) ) ;
                        draw_line ( x, 282,
                                    x, 297 - (curr_data.level_Y[i] / 2) ) ;
                        x++ ;
                    }
                } /* END FOR */

            if ( curr_data.level_step_length[i] == 254 )
                if ( x < 630 )
                {
                    draw_line ( x, 250,
                                x, 265 - (curr_data.level_X[i] / 2) ) ;
                    draw_line ( x, 282,
                                x, 297 - (curr_data.level_Y[i] / 2) ) ;
                    x++ ;
                }

            if ( curr_data.level_step_length[i] == 254 )  i = 0 ;
            else  i++ ;
        } /* END WHILE */

        /* Draw detune vector */
        x = 100 ;
        i = 0 ;
        while (( x < 630 ) && ( curr_data.detune_step_length[i] != 255 ) &&
               ( i < 50 ))
        {
            if ( curr_data.detune_step_length[i] < 254 )
                for ( k = 0 ; k < curr_data.detune_step_length[i] + 1 ; k++ )
                {
                    if ( x < 630 )
                    {
                        draw_line ( x, 314,
                                    x, 329 - (curr_data.detune_X[i] / 2) ) ;
                        draw_line ( x, 346,
                                    x, 362 - (curr_data.detune_Y[i] / 2) ) ;
                        x++ ;
                    }
                } /* END FOR */

            if ( curr_data.detune_step_length[i] == 254 )
                if ( x < 630 )
                {
                    draw_line ( x, 314,
                                x, 329 - (curr_data.detune_X[i] / 2) ) ;
                    draw_line ( x, 346,
                                x, 362 - (curr_data.detune_Y[i] / 2) ) ;
                    x++ ;
                }

            if ( curr_data.detune_step_length[i] == 254 )  i = 0 ;
            else  i++ ;
        } /* END WHILE */
        vswr_mode ( handle, 1 ) ;
        m_point_on ;
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "draw_vgraph" */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "init_but_flag"                                             */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function sets the selected button info to not pressed. */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "flag" : pointer to button info.                            */
/*--------------------------------------------------------------------------*/

    init_but_flag ( flag )
    hilite_type *flag ;
    {
        flag->now = FALSE ;
        flag->prev = FALSE ;
    }

/*------------------------------------*/
/*  END OF FUNCTION:  "init_but_flag" */
/*------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "check_but_stat"                                            */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function checks the current and previous status of the */
/*              indicated button.                                           */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "rect"   : area which defines the button.                   */
/*              "hilite" : pointer to button status.                        */
/*--------------------------------------------------------------------------*/

    check_but_stat ( rect, hilite )
    rect_type rect ;
    hilite_type *hilite ;
    {
        vswr_mode ( handle, 3 ) ;
        hilite->now = ( mx >= rect.left ) && ( mx <= rect.right ) &&
                      ( my >= rect.top )  && ( my <= rect.bottom ) &&
                     (( mbut & 0x01 ) == 0x01 ) ;

        if ( hilite->now && !hilite->prev )
        {
            hilite->prev = TRUE ;
            m_point_off ;
            vsf_interior ( handle, 1 ) ;
            v_rfbox ( handle, &rect ) ;
            m_point_on ;
        }
        else if ( !hilite->now && hilite->prev )
        {
            hilite->prev = FALSE ;
            m_point_off ;
            vsf_interior ( handle, 1 ) ;
            v_rfbox ( handle, &rect ) ;
            m_point_on ;
        }
        vswr_mode ( handle, 1 ) ;
    }

/*-------------------------------------*/
/*  END OF FUNCTION:  "check_but_stat" */
/*-------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_common"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the common element parameters           */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "element" : element to display, 0 to 3.                     */
/*--------------------------------------------------------------------------*/

    char com_text[15][19] = { "Wave..............", "Velocity response.",
                              "A.T. response.....", "LFO wave..........",
                              "LFO speed.........", "LFO delay.........",
                              "LFO rate..........", "LFO AM depth......",
                              "LFO PM depth......", "AM to carrier.....",
                              "AM to modulator...", "PM to carrier.....",
                              "PM to modulator...", "Pitch shift.......",
                              "Envelope type....." } ;

    draw_common ( element )
    int element ;
    {
        char text[20], i ;

        /* STORE AND DISPLAY EDIT MODE */
        curr_edit = element ;
        strcpy ( text, "EDIT ELEMENT x" ) ;
        text[13] = 65 + element ;
        vst_effects ( handle, 1 ) ;
        text_out ( 10, 224, text ) ;
        vst_effects ( handle, 0 ) ;

        /* DISPLAY OTHER INFO */
        for ( i = 0 ; i < 9 ; i++ )
            text_out ( 10, y_vals[i], com_text[i] ) ;

        if (( element == 1 ) || ( element == 3 ))
            for ( i = 9 ; i < 13 ; i++ )
                text_out ( 10, y_vals[i], com_text[i] ) ;

        for ( i = 13 ; i < 15 ; i++ )
            text_out ( 10, y_vals[i], com_text[i] ) ;
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "draw_common" */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "show_env"                                                  */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the element envelope parameters         */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "x" : x position at which to display.                       */
/*--------------------------------------------------------------------------*/

    char env_text[14][16] = {   "Level..........", "Temperament....",
                                "Detune.........", "Level scaling..",
                                "Rate scaling...", "Envelope delay.",
                                "Initial level..", "Attack rate....",
                                "Attack level...", "Decay 1 rate...",
                                "Decay 1 level..", "Decay 2 rate...",
                                "Decay 2 level..", "Release rate..." } ;

    show_env ( x ) 
    int x ;
    {
        int i ;
        for ( i = 0 ; i < 14 ; i++ )
            text_out ( x, y_vals[i], env_text[i] ) ;
    }

/*-------------------------------*/
/*  END OF FUNCTION:  "show_env" */
/*-------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_awm_element"                                          */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the static elements for editing AWM.    */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "element" : element to display, 0 to 3.                     */
/*--------------------------------------------------------------------------*/

    draw_awm_element ( element )
    int element ;
    {
        m_point_off ;
        clear_work_area () ;
        draw_common ( element ) ;
        show_env ( 210 ) ;
        m_point_on ;
    }

/*---------------------------------------*/
/*  END OF FUNCTION:  "draw_awm_element" */
/*---------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_fm_element"                                           */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the static elements for editing FM.     */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: "element" : element to display, 0 to 3.                     */
/*--------------------------------------------------------------------------*/

draw_fm_element ( element )
int element ;
{
    int x, i ;

        m_point_off ;
        clear_work_area () ;
        draw_common ( element ) ;
        text_out ( 210, 224, "CARRIER" ) ;
        text_out ( 410, 224, "MODULATOR" ) ;

        x = 210 ;
        for ( i = 0 ; i < 2 ; i++ )
        {
            show_env ( x ) ;
            text_out ( x, 344, "Fixed freq....." ) ;
            text_out ( x, 352, "Waveform......." ) ;
            text_out ( x, 360, "Harmonics......" ) ;
            x = 410 ;
        }
        text_out ( x, 368, "Feedback......." ) ;
        m_point_on ;
    }

/*--------------------------------------*/
/*  END OF FUNCTION:  "draw_fm_element" */
/*--------------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_vector"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the static elements for vector editing. */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    draw_vector ()
    {
        char text[20] ;

        m_point_off ;
        clear_work_area () ;

        curr_edit = 4 ;
        strcpy ( text, "EDIT VECTOR" ) ;
        vst_effects ( handle, 1 ) ;
        text_out ( 10, 224, text ) ;
        vst_effects ( handle, 0 ) ;

        text_out ( 10, 254, "Level X:" ) ;
        text_out ( 10, 286, "Level Y:" ) ;
        text_out ( 10, 318, "Detune X:" ) ;
        text_out ( 10, 350, "Detune Y:" ) ;
        m_point_on ;
    }

/*----------------------------------*/
/*  END OF FUNCTION:  "draw_vector" */
/*----------------------------------*/


/*--------------------------------------------------------------------------*/
/*  FUNCTION:   "draw_global"                                               */
/*--------------------------------------------------------------------------*/
/*  PURPOSE:    This function draws the static elements for editing voice   */
/*              voice common data.                                          */
/*--------------------------------------------------------------------------*/
/*  PARAMETERS: None.                                                       */
/*--------------------------------------------------------------------------*/

    char glob_text[16][22] =
        {  "Voice name...........", "Effect type..........",
           "Effect depth.........", "Elements.............",
           "Mod wheel AM.........", "Mod wheel PM.........",
           "After-touch AM.......", "After-touch PM.......",
           "After-touch level....", "After-touch pitch....",
           "Envelope delay rate..", "Common attack rate...",
           "Common release rate..", "Pitch bend range.....",
           "MIDI channel.........", "Synth................" } ;

    draw_global ()
    {
        char text[20], i ;

        m_point_off ;
        clear_work_area () ;

        curr_edit = 5 ;
        strcpy ( text, "EDIT GLOBAL" ) ;
        vst_effects ( handle, 1 ) ;
        text_out ( 10, 224, text ) ;
        vst_effects ( handle, 0 ) ;

        for ( i = 0 ; i < 16 ; i++ )
            text_out ( 10, y_vals[i], glob_text[i] ) ;

        m_point_on ;
    }

/*-----------------------------------*/
/*  END OF FUNCTION:  "draw_global"  */
/*-----------------------------------*/

