/*---------------------------------------------------------------------------*/
/* File:     TYPES.H                                                         */
/* Author:   R. P. Hanson                                                    */
/* Copyright:R. P. Hanson 1996                                               */
/*           Lothar Bittner is now the authorised custodian of the Atari ST  */
/*           version of this program.                                        */
/*---------------------------------------------------------------------------*/
/* Contents: Data types as used by SY-22 voice editor.                       */
/*---------------------------------------------------------------------------*/
/* Notes:    All data elements that refer to level are inversely scaled,     */
/*           i.e. 0 is maximum level and 127 is minimum level. When          */
/*           level is displayed on the screen, the logic is reversed such    */
/*           that 0 is minimum and 127 is maximum.                           */
/*---------------------------------------------------------------------------*/


/*----------------------------------------------------------------*/
/* Type:  "env_type"                                              */
/* Usage: All voice elements that require an envelope definition. */
/*----------------------------------------------------------------*/

typedef struct env_tag
    {
        unsigned char   delay_switch ;  /* Selects if envelope is delayed */
        unsigned char   attack_rate ;   /* Attack rate                    */
        unsigned char   drate_1 ;       /* Decay rate 1                   */
        unsigned char   env_peak ;      /* Envelope peak - must be set to */
                                        /* the envelope step that has the */
                                        /* highest level                  */
        unsigned char   drate_2 ;       /* Decay rate 2                   */
        unsigned char   release_rate ;  /* Release rate                   */
        unsigned char   initial_level ; /* Initial level (env. step 0)    */
        unsigned char   attack_level ;  /* Attack level  (env. step 1)    */
        unsigned char   dlevel_1 ;      /* Decay level 1 (env. step 2)    */
        unsigned char   dlevel_2 ;      /* Decay level 2 (env. step 3)    */
    } env_type ;


/*-------------------------------------------------------*/
/* Type:  "osc_type"                                     */
/* Usage: All elements common to both FM and AWM voices. */
/*-------------------------------------------------------*/

typedef struct osc_tag
    {
        unsigned char   fixed_frequency ;  /* FM fixed frequency flag */
        unsigned char   waveform ;         /* FM waveform             */
        unsigned char   frequency ;        /* FM frequency            */
        unsigned char   level ;            /* Element level           */
        unsigned char   temperament ;      /* Element temperament     */
        unsigned char   detune ;           /* Element detune          */
        unsigned char   level_scaling ;    /* Envelope level scaling  */
        unsigned char   rate_scaling ;     /* Envelope rate scaling   */
        env_type        envelope_def ;     /* Envelope definition     */
    } osc_type ;


/*--------------------------------------------------------------------------*/
/* Type:  "element_type"                                                    */
/* Usage: A single element. This data type is used for FM and AWM elements, */
/*        which means that some of the elements are redundant for an AWM    */
/*        element.                                                          */
/*--------------------------------------------------------------------------*/

typedef struct element_tag
    {
        unsigned char   wave ;          /* Wave source (AWM: 0 to 127)    */
                                        /*             (FM:  0 to 255)    */
        unsigned char   pitch_shift ;   /* Pitch shift (-12 to + 12)      */
        char            vel_response ;  /* Velocity response              */
        char            at_response ;   /* Aftertouch sensitivity         */
        unsigned char   LFO_wave ;      /* LFO waveform                   */
        unsigned char   LFO_speed ;     /* LFO speed                      */
        unsigned char   LFO_delay ;     /* Delay before LFO comes in      */
        unsigned char   LFO_rate ;      /* Rate at which LFO comes in     */
        unsigned char   LFO_AM_depth ;  /* Depth of tremolo               */
        unsigned char   AM_carrier ;    /* Assign tremolo to FM carrier   */
        unsigned char   AM_modulator ;  /* Assign tremolo to FM modulator */
        unsigned char   LFO_PM_depth ;  /* Depth of vibrato               */
        unsigned char   PM_carrier ;    /* Assign vibrato to FM carrier   */
        unsigned char   PM_modulator ;  /* Assign vibrato to FM modulator */
        unsigned char   envelope ;      /* Preset envelope number         */
        unsigned char   pan ;           /* Pan position                   */
        unsigned char   feedback ;      /* FM feedback                    */
        osc_type        osc_1 ;         /* FM modulator                   */
        osc_type        osc_2 ;         /* FM carrier / AWM oscillator    */
    } element_type ;


/*-------------------------------------------------------------------------*/
/* Global:"curr_data", "last_data"                                         */
/* Usage: Holds voice data for current and previous iteration. "last_data" */
/*        is compared against "curr_data" to determine what to update.     */
/*-------------------------------------------------------------------------*/

 struct vd_tag
    {
        unsigned char   unknown_1 ;       /* Unknown purpose              */
        unsigned char   unknown_2 ;       /* Unknown purpose              */
        unsigned char   effect ;          /* Effect depth                 */
        unsigned char   type ;            /* Effect type                  */
        char            name[9] ;         /* Voice name, null terminated  */
        unsigned char   num_elements ;    /* Number of elements in voice  */
        unsigned char   pitch_bend ;      /* Pitch bend range             */
        unsigned char   at_am_assign ;    /* Assign aftertouch to tremolo */
        unsigned char   at_pm_assign ;    /* Assign aftertouch to vibrato */
        unsigned char   at_lvl_assign ;   /* Assign aftertouch to volume  */
        unsigned char   mod_am_assign ;   /* Assign mod wheel to tremolo  */
        unsigned char   mod_pm_assign ;   /* Assign mod wheel to vibrato  */
        char            at_bend ;         /* Aftertouch bend range        */
        unsigned char   env_delay ;       /* Envelope delay               */
        char            attack_rate ;     /* Global attack rate modifier  */
        char            release_rate ;    /* Global release rate modifier */
        element_type    element[4] ;      /* Individual element data      */
        unsigned char   v_level_rate ;    /* Level vector rate            */
        unsigned char   v_detune_rate ;   /* Detune vector rate           */
        unsigned char   level_step_length[50] ; /*  -\                    */
        unsigned char   level_X[50] ;     /*         |  Level vector      */
        unsigned char   level_Y[50] ;     /*        -/                    */
        unsigned char   detune_step_length[50] ; /* -\                    */
        unsigned char   detune_X[50] ;    /*         |  Detune vector     */
        unsigned char   detune_Y[50] ;    /*        -/                    */
        unsigned char   _null_ ;          /* Always 0                     */
        unsigned char   checksum ;        /* Checksum                     */
    } curr_data, last_data ;


/*-----------------------------------------------*/
/* Defines:"TRUE", "FALSE"                       */
/* Usage:  To make code more readable in places! */
/*-----------------------------------------------*/

    #define TRUE    1
    #define FALSE   0
